@@ -3,7 +3,6 @@
 # Author: Gilles Louppe <g.louppe@gmail.com>
 # License: BSD 3 clause
 
-from __future__ import division
 
 import itertools
 import numbers
@@ -14,8 +13,6 @@ from warnings import warn
 from .base import BaseEnsemble, _partition_estimators
 from ..base import ClassifierMixin, RegressorMixin
 from ..utils._joblib import Parallel, delayed
-from ..externals.six import with_metaclass
-from ..externals.six.moves import zip
 from ..metrics import r2_score, accuracy_score
 from ..tree import DecisionTreeClassifier, DecisionTreeRegressor
 from ..utils import check_random_state, check_X_y, check_array, column_or_1d
@@ -184,7 +181,7 @@ def _parallel_predict_regression(estimators, estimators_features, X):
                                               estimators_features))
 
 
-class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
+class BaseBagging(BaseEnsemble, metaclass=ABCMeta):
     """Base class for Bagging meta-estimator.
 
     Warning: This class should not be used directly. Use derived classes
@@ -204,7 +201,7 @@ class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
                  n_jobs=None,
                  random_state=None,
                  verbose=0):
-        super(BaseBagging, self).__init__(
+        super().__init__(
             base_estimator=base_estimator,
             n_estimators=n_estimators)
 
@@ -243,6 +240,9 @@ class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
         """
         return self._fit(X, y, self.max_samples, sample_weight=sample_weight)
 
+    def _parallel_args(self):
+        return {}
+
     def _fit(self, X, y, max_samples=None, max_depth=None, sample_weight=None):
         """Build a Bagging ensemble of estimators from the training
            set (X, y).
@@ -365,7 +365,8 @@ class BaseBagging(with_metaclass(ABCMeta, BaseEnsemble)):
         seeds = random_state.randint(MAX_INT, size=n_more_estimators)
         self._seeds = seeds
 
-        all_results = Parallel(n_jobs=n_jobs, verbose=self.verbose)(
+        all_results = Parallel(n_jobs=n_jobs, verbose=self.verbose,
+                               **self._parallel_args())(
             delayed(_parallel_build_estimators)(
                 n_estimators[i],
                 self,
@@ -563,7 +564,7 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
                  random_state=None,
                  verbose=0):
 
-        super(BaggingClassifier, self).__init__(
+        super().__init__(
             base_estimator,
             n_estimators=n_estimators,
             max_samples=max_samples,
@@ -578,7 +579,7 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
 
     def _validate_estimator(self):
         """Check the estimator and set the base_estimator_ attribute."""
-        super(BaggingClassifier, self)._validate_estimator(
+        super()._validate_estimator(
             default=DecisionTreeClassifier())
 
     def _set_oob_score(self, X, y):
@@ -687,7 +688,8 @@ class BaggingClassifier(BaseBagging, ClassifierMixin):
         n_jobs, n_estimators, starts = _partition_estimators(self.n_estimators,
                                                              self.n_jobs)
 
-        all_proba = Parallel(n_jobs=n_jobs, verbose=self.verbose)(
+        all_proba = Parallel(n_jobs=n_jobs, verbose=self.verbose,
+                             **self._parallel_args())(
             delayed(_parallel_predict_proba)(
                 self.estimators_[starts[i]:starts[i + 1]],
                 self.estimators_features_[starts[i]:starts[i + 1]],
@@ -932,7 +934,7 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
                  n_jobs=None,
                  random_state=None,
                  verbose=0):
-        super(BaggingRegressor, self).__init__(
+        super().__init__(
             base_estimator,
             n_estimators=n_estimators,
             max_samples=max_samples,
@@ -987,7 +989,7 @@ class BaggingRegressor(BaseBagging, RegressorMixin):
 
     def _validate_estimator(self):
         """Check the estimator and set the base_estimator_ attribute."""
-        super(BaggingRegressor, self)._validate_estimator(
+        super()._validate_estimator(
             default=DecisionTreeRegressor())
 
     def _set_oob_score(self, X, y):
